package com.vcarecity.cvs.script;

import lombok.extern.slf4j.Slf4j;

import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;

/**
 * @author Kerry on 19/11/25
 * <p>
 * <p>
 * 创建表分区 datetime
 */

@Slf4j
public class BuildPartitionSQL {

    private static final DateTimeFormatter DATE_FORMATTER = DateTimeFormatter.ofPattern("yyyy-MM-dd");


    public static void main(String[] args) {
        BuildPartitionSQL buildPartitionSQL = new BuildPartitionSQL();
        buildPartitionSQL.partitionFirstSql("T_ALARM_LAST", "stamp", "2016-03-29", "2020-01-01", PartitionType.MONTH);
    }


    public String partitionFirstSql(String table, String field, String from, String to, PartitionType partitionType) {
        return partitionFirstSql(table, field, LocalDate.parse(from, DATE_FORMATTER), LocalDate.parse(to, DATE_FORMATTER), partitionType);
    }

    public String partitionFirstSql(String table, String field, LocalDate from, LocalDate to, PartitionType partitionType) {

        final String[] strings = partitionSql(from, to, partitionType);
        final String join = String.join(",\n", strings);

        String pre = "ALTER TABLE " + table + " PARTITION BY RANGE (TO_DAYS(" + field + ")) (";


        logger.info("--------------------------first create table partition SQL.--------------------------");

        String s = pre + "\n" + join + "\n" + ")";
        System.out.println(s);

        logger.info("--------------------------first create table partition SQL.--------------------------");
        return s;
    }

    public String[] partitionSql(LocalDate from, LocalDate to, PartitionType partitionType) {
        LocalDate index = from;
        to = plusDateByType(to, partitionType);

        List<String> partitionSqls = new ArrayList<>();
        while (index.isBefore(to)) {
            String pName = partitionType.getFormatter().format(index);
            index = plusDateByType(index, partitionType);
            final String format = DATE_FORMATTER.format(index);
            String alter = "PARTITION p" + pName + " VALUES LESS THAN (TO_DAYS('" + format + "')) ENGINE = INNODB";
            partitionSqls.add(alter);
        }
        return partitionSqls.toArray(new String[]{});
    }

    private enum PartitionType {
        /**
         * 天
         */
        DAY("yyyyMMdd"),
        /**
         * 月
         */
        MONTH("yyyyMM");
        DateTimeFormatter formatter;

        PartitionType(String format) {
            this.formatter = DateTimeFormatter.ofPattern(format);
        }

        public DateTimeFormatter getFormatter() {
            return formatter;
        }
    }

    private static LocalDate plusDateByType(LocalDate localDate, PartitionType partitionType) {
        switch (partitionType) {
            case DAY:
                return localDate.plusDays(1L);
            case MONTH:
                LocalDate date = localDate.plusMonths(1L);
                return date.minusDays(date.getDayOfMonth() - 1);
            default:
                break;
        }
        throw new RuntimeException(partitionType.name());
    }
}
