package com.vcarecity.cvs.service.impl;

import com.google.inject.Inject;
import com.zaxxer.hikari.HikariDataSource;
import lombok.extern.slf4j.Slf4j;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

/**
 * @author Kerry on 19/11/25
 */

@Slf4j
public class OracleQueryServiceImpl extends AbstractSQLQueryService {

    private static final String PAGE_SQL = "select * from (select ROWNUM as RN, T.* from %s T where ROWNUM <= ?) A where A.RN > ?";

    @Inject
    public OracleQueryServiceImpl(HikariDataSource dataSource) {
        super(dataSource);
    }

    @Override
    protected String getQuerySql(String table) {
        return String.format(PAGE_SQL, table);
    }

    @Override
    protected PreparedStatement getPreparedStatement(Connection connection, String sql, int page, int pageCount) throws SQLException {

        logger.debug("start query: [{}]. pageInfo = {}/{}. time = {}", sql, (1 + page) * pageCount, page * pageCount, System.currentTimeMillis());

        PreparedStatement preparedStatement = connection.prepareStatement(sql);
        preparedStatement.setInt(1, page * pageCount);
        preparedStatement.setInt(2, (page - 1) * pageCount);
        return preparedStatement;
    }

    @Override
    protected List<ColumnIndexName> getColumnWithObject(ResultSetMetaData metaData) throws SQLException {
        final int columnCount = metaData.getColumnCount();
        List<ColumnIndexName> res = new ArrayList<>(columnCount);
        for (int i = 0; i < columnCount; i++) {
            String name = metaData.getColumnName(i + 1);
            if ("RN".equalsIgnoreCase(name)) {
                continue;
            }
            res.add(new ColumnIndexName(i, name));
        }
        return res;
    }
}
